##################### generated by xml-casa (v2) from clearcal.xml ##################
##################### 0e74985bc220c252fb4057cc1a73cb9d ##############################
from __future__ import absolute_import
from casashell.private.stack_manip import find_local as __sf__
from casashell.private.stack_manip import find_frame as _find_frame
from casatools.typecheck import validator as _pc
from casatools.coercetype import coerce as _coerce
from casatasks import clearcal as _clearcal_t
from collections import OrderedDict
import sys
import os

def static_var(varname, value):
    def decorate(func):
        setattr(func, varname, value)
        return func
    return decorate

class _clearcal:
    """
    clearcal ---- Re-initializes the calibration for a visibility data set

    
    Clearcal reinitializes the calibration columns in a measurement
    set. Specificially, it will set the MODEL_DATA column (if present) to
    unity in total intensity and zero in polarization, and it will set the
    CORRECTED_DATA column to the original (observed) DATA in the DATA
    column.  Use the field and spw parameters to select which data to
    initialize.  If the dataset does not yet have the scratch columns,
    they will be created (MODEL_DATA only if addmodel=True) and initilized
    for the whole dataset (field, spw, and intent will be ignored in this
    case).

    --------- parameter descriptions ---------------------------------------------

    vis      Name of input visibility file (MS)
             Default: none
             
                Example: vis='ngc5921.ms'
    field    Select field using field id(s) or field name(s)
             default: '' (all fields)
             
             Use 'go listobs' to obtain the list id's or
             names. If field string is a non-negative integer,
             it is assumed a field index,  otherwise, it is
             assumed a field name.
             
                Examples:
                field='0~2'; field ids 0,1,2
                field='0,4,5~7'; field ids 0,4,5,6,7
                field='3C286,3C295'; field named 3C286 and
                3C295
                field = '3,4C*'; field id 3, all names
                starting with 4C
    spw      Select spectral window/channels
             
             Examples:
             spw='0~2,4'; spectral windows 0,1,2,4 (all
             channels)
             spw='<2';  spectral windows less than 2
             (i.e. 0,1)
             spw='0:5~61'; spw 0, channels 5 to 61,
             INCLUSIVE
             spw='*:5~61'; all spw with channels 5 to 61
             spw='0,10,3:3~45'; spw 0,10 all channels, spw
             3, channels 3 to 45.
             spw='0~2:2~6'; spw 0,1,2 with channels 2
             through 6 in each.
             spw='0:0~10;15~60'; spectral window 0 with
             channels 0-10,15-60. (NOTE ';' to separate
             channel selections)
             spw='0:0~10^2,1:20~30^5'; spw 0, channels
             0,2,4,6,8,10, spw 1, channels 20,25,30 
             type 'help par.selection' for more examples.
    intent   Select observing intent
             default: '' (no selection by intent)
             
                Example: intent='*BANDPASS*'  (selects data
                labelled with BANDPASS intent)
    addmodel add MODEL_DATA along with CORRECTED_DATA?
             Default: False (model will not be added)
             Options: False|True
             
             If False, it will add/reset only CORRECTED_DATA,
             model visibilities will then be evaluated when
             needed.
    [1;42mRETURNS[1;m     void

    --------- examples -----------------------------------------------------------

    
    
    For more information, see the task pages of clearcal in CASA Docs:
    
    https://casa.nrao.edu/casadocs/
    
    


    """

    _info_group_ = """calibration"""
    _info_desc_ = """Re-initializes the calibration for a visibility data set"""

    __schema = {'vis': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'field': {'type': 'cStr'}, 'spw': {'type': 'cStr'}, 'intent': {'type': 'cStr'}, 'addmodel': {'type': 'cBool'}}

    def __init__(self):
        self.__stdout = None
        self.__stderr = None
        self.__root_frame_ = None

    def __globals_(self):
        if self.__root_frame_ is None:
            self.__root_frame_ = _find_frame( )
            assert self.__root_frame_ is not None, "could not find CASAshell global frame"
        return self.__root_frame_

    def __to_string_(self,value):
        if type(value) is str:
            return "'%s'" % value
        else:
            return str(value)

    def __validate_(self,doc,schema):
        return _pc.validate(doc,schema)

    #--------- return nonsubparam values ----------------------------------------------

    def __addmodel_dflt( self, glb ):
        return False

    def __addmodel( self, glb ):
        if 'addmodel' in glb: return glb['addmodel']
        return False

    def __vis_dflt( self, glb ):
        return ''

    def __vis( self, glb ):
        if 'vis' in glb: return glb['vis']
        return ''

    def __spw_dflt( self, glb ):
        return ''

    def __spw( self, glb ):
        if 'spw' in glb: return glb['spw']
        return ''

    def __field_dflt( self, glb ):
        return ''

    def __field( self, glb ):
        if 'field' in glb: return glb['field']
        return ''

    def __intent_dflt( self, glb ):
        return ''

    def __intent( self, glb ):
        if 'intent' in glb: return glb['intent']
        return ''



    #--------- return inp/go default --------------------------------------------------


    #--------- return subparam values -------------------------------------------------


    #--------- subparam inp output ----------------------------------------------------
    def __vis_inp(self):
        out = self.__stdout or sys.stdout
        description = 'Name of input visibility file (MS)'
        value = self.__vis( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'vis': value},{'vis': self.__schema['vis']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-8.8s = %s%-23.22s%s # %-.60s\n' % ('vis',pre,self.__to_string_(value),post,description))
    def __field_inp(self):
        out = self.__stdout or sys.stdout
        description = 'Select field using field id(s) or field name(s)'
        value = self.__field( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'field': value},{'field': self.__schema['field']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-8.8s = %s%-23.22s%s # %-.60s\n' % ('field',pre,self.__to_string_(value),post,description))
    def __spw_inp(self):
        out = self.__stdout or sys.stdout
        description = 'Select spectral window/channels'
        value = self.__spw( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'spw': value},{'spw': self.__schema['spw']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-8.8s = %s%-23.22s%s # %-.60s\n' % ('spw',pre,self.__to_string_(value),post,description))
    def __intent_inp(self):
        out = self.__stdout or sys.stdout
        description = 'Select observing intent'
        value = self.__intent( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'intent': value},{'intent': self.__schema['intent']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-8.8s = %s%-23.22s%s # %-.60s\n' % ('intent',pre,self.__to_string_(value),post,description))
    def __addmodel_inp(self):
        out = self.__stdout or sys.stdout
        description = 'Add MODEL_DATA scratch column'
        value = self.__addmodel( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'addmodel': value},{'addmodel': self.__schema['addmodel']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-8.8s = %s%-23.22s%s # %-.60s\n' % ('addmodel',pre,self.__to_string_(value),post,description))

    #--------- global default implementation-------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def set_global_defaults(self):
        self.set_global_defaults.state['last'] = self
        glb = self.__globals_( )
        if 'field' in glb: del glb['field']
        if 'intent' in glb: del glb['intent']
        if 'vis' in glb: del glb['vis']
        if 'addmodel' in glb: del glb['addmodel']
        if 'spw' in glb: del glb['spw']


    #--------- inp function -----------------------------------------------------------
    def inp(self):
        self.__vis_inp( )
        self.__field_inp( )
        self.__spw_inp( )
        self.__intent_inp( )
        self.__addmodel_inp( )

    #--------- tget function ----------------------------------------------------------
    def tget(self,file=None):
        from .stack_manip import find_frame
        from runpy import run_path
        filename = None
        if file is None:
            if os.path.isfile("clearcal.last"):
                filename = "clearcal.last"
        elif isinstance(file, str):
            if os.path.isfile(file):
                filename = file
        if filename is not None:
            glob = find_frame( )
            newglob = run_path( filename, init_globals={ } )
            for i in newglob:
                glob[i] = newglob[i]

    def __call__( self, vis=None, field=None, spw=None, intent=None, addmodel=None ):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s
        _prefile = os.path.realpath('clearcal.pre')
        _postfile = os.path.realpath('clearcal.last')
        _return_result_ = None
        _arguments = [vis,field,spw,intent,addmodel]
        _invocation_parameters = OrderedDict( )
        if any(map(lambda x: x is not None,_arguments)):
            # invoke python style
            # set the non sub-parameters that are not None
            local_global = { }
            if vis is not None: local_global['vis'] = vis
            if field is not None: local_global['field'] = field
            if spw is not None: local_global['spw'] = spw
            if intent is not None: local_global['intent'] = intent
            if addmodel is not None: local_global['addmodel'] = addmodel

            # the invocation parameters for the non-subparameters can now be set - this picks up those defaults
            _invocation_parameters['vis'] = self.__vis( local_global )
            _invocation_parameters['field'] = self.__field( local_global )
            _invocation_parameters['spw'] = self.__spw( local_global )
            _invocation_parameters['intent'] = self.__intent( local_global )
            _invocation_parameters['addmodel'] = self.__addmodel( local_global )

            # the sub-parameters can then be set. Use the supplied value if not None, else the function, which gets the appropriate default
            

        else:
            # invoke with inp/go semantics
            _invocation_parameters['vis'] = self.__vis( self.__globals_( ) )
            _invocation_parameters['field'] = self.__field( self.__globals_( ) )
            _invocation_parameters['spw'] = self.__spw( self.__globals_( ) )
            _invocation_parameters['intent'] = self.__intent( self.__globals_( ) )
            _invocation_parameters['addmodel'] = self.__addmodel( self.__globals_( ) )
        try:
            with open(_prefile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-8s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#clearcal( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: pass
        try:
            _return_result_ = _clearcal_t( _invocation_parameters['vis'],_invocation_parameters['field'],_invocation_parameters['spw'],_invocation_parameters['intent'],_invocation_parameters['addmodel'] )
        except: _return_result_ = False
        try:
            os.rename(_prefile,_postfile)
        except: pass
        return _return_result_

clearcal = _clearcal( )

