##################### generated by xml-casa (v2) from listvis.xml ###################
##################### 4212845eac8fe9322530e5b50126b2ec ##############################
from __future__ import absolute_import
from casashell.private.stack_manip import find_local as __sf__
from casashell.private.stack_manip import find_frame as _find_frame
from casatools.typecheck import validator as _pc
from casatools.coercetype import coerce as _coerce
from casatasks import listvis as _listvis_t
from collections import OrderedDict
import sys
import os

def static_var(varname, value):
    def decorate(func):
        setattr(func, varname, value)
        return func
    return decorate

class _listvis:
    """
    listvis ---- List measurement set visibilities.

    
    
    This task lists measurement set visibility data under a number of
    input selection conditions.  The measurement set data columns that
    can be listed are: the raw data, float_data, corrected data, model data,
    and residual (corrected - model) data.
    
    The output table format is dynamic.  Field, Spectral Window, and
    Channel columns are not displayed if the column contents are uniform.
    For example, if "spw = '1'" is specified, the spw column will not be
    displayed.  When a column is not displayed, a message is sent to the
    logger and terminal indicating that the column values are uniform and
    listing the uniform value.
    
    Table column descriptions:
    
    COLUMN NAME       DESCRIPTION
    -----------       -----------
    Date/Time         Time stamp of data sample (YYMMDD/HH:MM:SS UT)
    Intrf             Interferometer baseline (antenna names)
    UVDist            uv-distance (units of wavelength)
    Fld               Field ID (if more than 1)
    SpW               Spectral Window ID (if more than 1)
    Chn               Channel number (if more than 1)
    (Correlated       Correlated polarizations (eg: RR, LL, XY)
    polarization)     Sub-columns are: Amp, Phs, Wt, F
    Amp               Visibility amplitude
    Phs               Visibility phase (deg)
    Wt                Weight of visibility measurement
    F                 Flag: 'F' = flagged datum; ' ' = unflagged
    UVW               UVW coordinates (meters)
    
    
    Input Parameters:
    vis         Name of input visibility file
    default: none; example: vis='ngc5921.ms'
    
    options     List options: default = 'ap'
    Not yet implemented for suboptions
    
    datacolumn  Visibility file data column:
    default = 'data':  options are
    data, float_data, corrected, model, residual (corrected-model)
    
    field       Select data based on field id(s) or name(s)
    default: ''==>all; example: field='1'
    field='0~2' field ids inclusive from 0 to 2
    field='3C*' all field names starting with 3C
    
    spw         Select spectral window, channel to list
    default: '0:0' --> spw=0, chan=0
    spw='2:34' spectral window 2, channel 34
    
    selectdata  Toggle the following 7 selection parameters.
    default: False; example: selectdata=True
    If false, the following parameters are reset
    to default values.
    
    antenna     Select calibration data based on antenna
    default: ''-->all; examples:
    antenna = '5,6'; antenna index 5 and 6 solutions
    antenna = '05,06'; antenna names '05' and '06 solutions
    
    timerange   Select time range to list
    default: ''-->all; examples:
    timerange='10:37:50.1'; list data for this sampling interval
    timerange='<10:37:25'; list data before 10:37:25
    
    correlation Select polarization correlations to list
    default: ''-->all; examples:
    correlation='RR LL'; list RR and LL correlations
    correlation='XX XY'; list XX and XY correlations
    
    scan        Select scans to list
    default: ''-->all; examples:
    scan='2'; list scan 2
    scan='>2'; list scan numbers greater than 2
    
    feed        (not yet implemented)
    
    array       (not yet implemented)
    
    observation Select by observation ID.
    
    uvrange     Select baseline lengths to list.
    default: ''--> all; examples:
    uvrange='<5klambda'; less than 5 kilo-wavelengths
    Caution: Input units default to meters.
    Listed units are always wavelengths.
    
    average     (not yet implemented)
    
    showflags   (not yet implemented)
    
    pagerows    rows per page of listing
    default: 50; 0 --> do not paginate
    
    listfile    write output to disk; will not overwrite
    default: '' --> write to screen
    listfile = 'solutions.txt'
    
    

    --------- parameter descriptions ---------------------------------------------

    vis         Name of input visibility file
    options     List options: ap only
    datacolumn  Column to list: data, float_data, corrected, model, residual
    field       Field names or index to be listed
    spw         Spectral window channels
    selectdata  Other data selection parameters
    antenna     Antenna/baselines
    timerange   Time range
    correlation Correlations
    scan        Scan numbers
    feed        Multi-feed numbers (Not yet implemented)
    array       Array numbers (Not yet implemented)
    observation Select by observation ID(s)
    uvrange     uv range
    average     Averaging mode
    showflags   Show flagged data (Not yet implemented)
    pagerows    Rows per page
    listfile    Output file
    [1;42mRETURNS[1;m        void

    --------- examples -----------------------------------------------------------

    
    
    This task lists measurement set visibility data under a number of
    input selection conditions.  The measurement set data columns that
    can be listed are: the raw data, float_data, corrected data, model data,
    and residual (corrected - model) data.
    
    The output table format is dynamic.  Field, Spectral Window, and
    Channel columns are not displayed if the column contents are uniform.
    For example, if "spw = '1'" is specified, the spw column will not be
    displayed.  When a column is not displayed, a message is sent to the
    logger and terminal indicating that the column values are uniform and
    listing the uniform value.
    
    Table column descriptions:
    
    COLUMN NAME       DESCRIPTION
    -----------       -----------
    Date/Time         Time stamp of data sample (YYMMDD/HH:MM:SS UT)
    Intrf             Interferometer baseline (antenna names)
    UVDist            uv-distance (units of wavelength)
    Fld               Field ID (if more than 1)
    SpW               Spectral Window ID (if more than 1)
    Chn               Channel number (if more than 1)
    (Correlated       Correlated polarizations (eg: RR, LL, XY)
    polarization)     Sub-columns are: Amp, Phs, Wt, F
    Amp               Visibility amplitude
    Phs               Visibility phase (deg)
    Wt                Weight of visibility measurement
    F                 Flag: 'F' = flagged datum; ' ' = unflagged
    UVW               UVW coordinates (meters)
    
    
    Input Parameters:
    vis         Name of input visibility file
    default: none; example: vis='ngc5921.ms'
    
    options     List options: default = 'ap'
    Not yet implemented for suboptions
    
    datacolumn  Visibility file data column:
    default = 'data':  options are
    data, float_data, corrected, model, residual (corrected-model)
    
    field       Select data based on field id(s) or name(s)
    default: ''==>all; example: field='1'
    field='0~2' field ids inclusive from 0 to 2
    field='3C*' all field names starting with 3C
    
    spw         Select spectral window, channel to list
    default: '0:0' --> spw=0, chan=0
    spw='2:34' spectral window 2, channel 34
    
    selectdata  Toggle the following 7 selection parameters.
    default: False; example: selectdata=True
    If false, the following parameters are reset
    to default values.
    
    antenna     Select calibration data based on antenna
    default: ''-->all; examples:
    antenna = '5,6'; antenna index 5 and 6 solutions
    antenna = '05,06'; antenna names '05' and '06 solutions
    
    timerange   Select time range to list
    default: ''-->all; examples:
    timerange='10:37:50.1'; list data for this sampling interval
    timerange='<10:37:25'; list data before 10:37:25
    
    correlation Select polarization correlations to list
    default: ''-->all; examples:
    correlation='RR LL'; list RR and LL correlations
    correlation='XX XY'; list XX and XY correlations
    
    scan        Select scans to list
    default: ''-->all; examples:
    scan='2'; list scan 2
    scan='>2'; list scan numbers greater than 2
    
    feed        (not yet implemented)
    
    array       (not yet implemented)
    
    observation Select by observation ID(s).
    default: ''-->all;
    example: observation='0' (select obsID 0)
    
    uvrange     Select baseline lengths to list.
    default: ''--> all; examples:
    uvrange='<5klambda'; less than 5 kilo-wavelengths
    Caution: Input units default to meters.
    Listed units are always wavelengths.
    
    average     (not yet implemented)
    
    showflags   (not yet implemented)
    
    pagerows    rows per page of listing
    default: 50; 0 --> do not paginate
    
    listfile    write output to disk; will not overwrite
    default: '' --> write to screen
    listfile = 'solutions.txt'
    
    


    """

    _info_group_ = """information, analysis"""
    _info_desc_ = """List measurement set visibilities."""

    __schema = {'vis': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'options': {'type': 'cStr'}, 'datacolumn': {'type': 'cStr'}, 'field': {'type': 'cStr'}, 'spw': {'type': 'cStr'}, 'selectdata': {'type': 'cBool'}, 'antenna': {'type': 'cStr'}, 'timerange': {'type': 'cStr'}, 'correlation': {'type': 'cStr'}, 'scan': {'type': 'cStr'}, 'feed': {'type': 'cStr'}, 'array': {'type': 'cStr'}, 'observation': {'anyof': [{'type': 'cStr'}, {'type': 'cInt'}]}, 'uvrange': {'type': 'cStr'}, 'average': {'type': 'cStr'}, 'showflags': {'type': 'cBool'}, 'pagerows': {'type': 'cInt'}, 'listfile': {'type': 'cStr'}}

    def __init__(self):
        self.__stdout = None
        self.__stderr = None
        self.__root_frame_ = None

    def __globals_(self):
        if self.__root_frame_ is None:
            self.__root_frame_ = _find_frame( )
            assert self.__root_frame_ is not None, "could not find CASAshell global frame"
        return self.__root_frame_

    def __to_string_(self,value):
        if type(value) is str:
            return "'%s'" % value
        else:
            return str(value)

    def __validate_(self,doc,schema):
        return _pc.validate(doc,schema)

    #--------- return nonsubparam values ----------------------------------------------

    def __average_dflt( self, glb ):
        return ''

    def __average( self, glb ):
        if 'average' in glb: return glb['average']
        return ''

    def __listfile_dflt( self, glb ):
        return ''

    def __listfile( self, glb ):
        if 'listfile' in glb: return glb['listfile']
        return ''

    def __vis_dflt( self, glb ):
        return ''

    def __vis( self, glb ):
        if 'vis' in glb: return glb['vis']
        return ''

    def __selectdata_dflt( self, glb ):
        return False

    def __selectdata( self, glb ):
        if 'selectdata' in glb: return glb['selectdata']
        return False

    def __spw_dflt( self, glb ):
        return '*'

    def __spw( self, glb ):
        if 'spw' in glb: return glb['spw']
        return '*'

    def __pagerows_dflt( self, glb ):
        return int(50)

    def __pagerows( self, glb ):
        if 'pagerows' in glb: return glb['pagerows']
        return int(50)

    def __datacolumn_dflt( self, glb ):
        return 'data'

    def __datacolumn( self, glb ):
        if 'datacolumn' in glb: return glb['datacolumn']
        return 'data'

    def __showflags_dflt( self, glb ):
        return False

    def __showflags( self, glb ):
        if 'showflags' in glb: return glb['showflags']
        return False

    def __field_dflt( self, glb ):
        return ''

    def __field( self, glb ):
        if 'field' in glb: return glb['field']
        return ''

    def __options_dflt( self, glb ):
        return 'ap'

    def __options( self, glb ):
        if 'options' in glb: return glb['options']
        return 'ap'

    #--------- return non subparam/when values ---------------------------------------------
    def __observation( self, glb ):
        if 'observation' in glb: return glb['observation']
        return ''

    #--------- return inp/go default --------------------------------------------------
    def __antenna_dflt( self, glb ):
        if self.__selectdata( glb ) == bool(True): return ""
        return None
    def __scan_dflt( self, glb ):
        if self.__selectdata( glb ) == bool(True): return ""
        return None
    def __array_dflt( self, glb ):
        if self.__selectdata( glb ) == bool(True): return ""
        return None
    def __correlation_dflt( self, glb ):
        if self.__selectdata( glb ) == bool(True): return ""
        return None
    def __feed_dflt( self, glb ):
        if self.__selectdata( glb ) == bool(True): return ""
        return None
    def __uvrange_dflt( self, glb ):
        if self.__selectdata( glb ) == bool(True): return ""
        return None
    def __observation_dflt( self, glb ):
        if self.__selectdata( glb ) == bool(True): return ""
        return None
    def __timerange_dflt( self, glb ):
        if self.__selectdata( glb ) == bool(True): return ""
        return None

    #--------- return subparam values -------------------------------------------------
    def __antenna( self, glb ):
        if 'antenna' in glb: return glb['antenna']
        dflt = self.__antenna_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __timerange( self, glb ):
        if 'timerange' in glb: return glb['timerange']
        dflt = self.__timerange_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __correlation( self, glb ):
        if 'correlation' in glb: return glb['correlation']
        dflt = self.__correlation_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __scan( self, glb ):
        if 'scan' in glb: return glb['scan']
        dflt = self.__scan_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __feed( self, glb ):
        if 'feed' in glb: return glb['feed']
        dflt = self.__feed_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __array( self, glb ):
        if 'array' in glb: return glb['array']
        dflt = self.__array_dflt( glb )
        if dflt is not None: return dflt
        return ''
    def __uvrange( self, glb ):
        if 'uvrange' in glb: return glb['uvrange']
        dflt = self.__uvrange_dflt( glb )
        if dflt is not None: return dflt
        return ''

    #--------- subparam inp output ----------------------------------------------------
    def __vis_inp(self):
        out = self.__stdout or sys.stdout
        description = ''
        value = self.__vis( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'vis': value},{'vis': self.__schema['vis']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-14.14s = %s%-23.22s%s # %-.60s\n' % ('vis',pre,self.__to_string_(value),post,description))
    def __options_inp(self):
        out = self.__stdout or sys.stdout
        description = ''
        value = self.__options( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'options': value},{'options': self.__schema['options']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-14.14s = %s%-23.22s%s # %-.60s\n' % ('options',pre,self.__to_string_(value),post,description))
    def __datacolumn_inp(self):
        out = self.__stdout or sys.stdout
        description = ''
        value = self.__datacolumn( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'datacolumn': value},{'datacolumn': self.__schema['datacolumn']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-14.14s = %s%-23.22s%s # %-.60s\n' % ('datacolumn',pre,self.__to_string_(value),post,description))
    def __field_inp(self):
        out = self.__stdout or sys.stdout
        description = ''
        value = self.__field( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'field': value},{'field': self.__schema['field']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-14.14s = %s%-23.22s%s # %-.60s\n' % ('field',pre,self.__to_string_(value),post,description))
    def __spw_inp(self):
        out = self.__stdout or sys.stdout
        description = ''
        value = self.__spw( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'spw': value},{'spw': self.__schema['spw']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-14.14s = %s%-23.22s%s # %-.60s\n' % ('spw',pre,self.__to_string_(value),post,description))
    def __selectdata_inp(self):
        out = self.__stdout or sys.stdout
        description = ''
        value = self.__selectdata( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'selectdata': value},{'selectdata': self.__schema['selectdata']}) else ('\x1B[91m','\x1B[0m')
        out.write('\x1B[1m\x1B[47m%-14.14s =\x1B[0m %s%-23.22s%s # %-.60s\n' % ('selectdata',pre,self.__to_string_(value),post,description))
    def __antenna_inp(self):
        out = self.__stdout or sys.stdout
        if self.__antenna_dflt( self.__globals_( ) ) is not None:
             description = ''
             value = self.__antenna( self.__globals_( ) )
             (pre,post) = ('','') if self.__validate_({'antenna': value},{'antenna': self.__schema['antenna']}) else ('\x1B[91m','\x1B[0m')
             out.write('   \x1B[92m%-11.11s =\x1B[0m %s%-23.22s%s # %-.60s\n' % ('antenna',pre,self.__to_string_(value),post,description))
    def __timerange_inp(self):
        out = self.__stdout or sys.stdout
        if self.__timerange_dflt( self.__globals_( ) ) is not None:
             description = ''
             value = self.__timerange( self.__globals_( ) )
             (pre,post) = ('','') if self.__validate_({'timerange': value},{'timerange': self.__schema['timerange']}) else ('\x1B[91m','\x1B[0m')
             out.write('   \x1B[92m%-11.11s =\x1B[0m %s%-23.22s%s # %-.60s\n' % ('timerange',pre,self.__to_string_(value),post,description))
    def __correlation_inp(self):
        out = self.__stdout or sys.stdout
        if self.__correlation_dflt( self.__globals_( ) ) is not None:
             description = ''
             value = self.__correlation( self.__globals_( ) )
             (pre,post) = ('','') if self.__validate_({'correlation': value},{'correlation': self.__schema['correlation']}) else ('\x1B[91m','\x1B[0m')
             out.write('   \x1B[92m%-11.11s =\x1B[0m %s%-23.22s%s # %-.60s\n' % ('correlation',pre,self.__to_string_(value),post,description))
    def __scan_inp(self):
        out = self.__stdout or sys.stdout
        if self.__scan_dflt( self.__globals_( ) ) is not None:
             description = ''
             value = self.__scan( self.__globals_( ) )
             (pre,post) = ('','') if self.__validate_({'scan': value},{'scan': self.__schema['scan']}) else ('\x1B[91m','\x1B[0m')
             out.write('   \x1B[92m%-11.11s =\x1B[0m %s%-23.22s%s # %-.60s\n' % ('scan',pre,self.__to_string_(value),post,description))
    def __feed_inp(self):
        out = self.__stdout or sys.stdout
        if self.__feed_dflt( self.__globals_( ) ) is not None:
             description = ''
             value = self.__feed( self.__globals_( ) )
             (pre,post) = ('','') if self.__validate_({'feed': value},{'feed': self.__schema['feed']}) else ('\x1B[91m','\x1B[0m')
             out.write('   \x1B[92m%-11.11s =\x1B[0m %s%-23.22s%s # %-.60s\n' % ('feed',pre,self.__to_string_(value),post,description))
    def __array_inp(self):
        out = self.__stdout or sys.stdout
        if self.__array_dflt( self.__globals_( ) ) is not None:
             description = ''
             value = self.__array( self.__globals_( ) )
             (pre,post) = ('','') if self.__validate_({'array': value},{'array': self.__schema['array']}) else ('\x1B[91m','\x1B[0m')
             out.write('   \x1B[92m%-11.11s =\x1B[0m %s%-23.22s%s # %-.60s\n' % ('array',pre,self.__to_string_(value),post,description))
    def __observation_inp(self):
        out = self.__stdout or sys.stdout
        description = ''
        value = self.__observation( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'observation': value},{'observation': self.__schema['observation']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-14.14s = %s%-23.22s%s # %-.60s\n' % ('observation',pre,self.__to_string_(value),post,description))
    def __uvrange_inp(self):
        out = self.__stdout or sys.stdout
        if self.__uvrange_dflt( self.__globals_( ) ) is not None:
             description = ''
             value = self.__uvrange( self.__globals_( ) )
             (pre,post) = ('','') if self.__validate_({'uvrange': value},{'uvrange': self.__schema['uvrange']}) else ('\x1B[91m','\x1B[0m')
             out.write('   \x1B[92m%-11.11s =\x1B[0m %s%-23.22s%s # %-.60s\n' % ('uvrange',pre,self.__to_string_(value),post,description))
    def __average_inp(self):
        out = self.__stdout or sys.stdout
        description = ''
        value = self.__average( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'average': value},{'average': self.__schema['average']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-14.14s = %s%-23.22s%s # %-.60s\n' % ('average',pre,self.__to_string_(value),post,description))
    def __showflags_inp(self):
        out = self.__stdout or sys.stdout
        description = ''
        value = self.__showflags( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'showflags': value},{'showflags': self.__schema['showflags']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-14.14s = %s%-23.22s%s # %-.60s\n' % ('showflags',pre,self.__to_string_(value),post,description))
    def __pagerows_inp(self):
        out = self.__stdout or sys.stdout
        description = ''
        value = self.__pagerows( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'pagerows': value},{'pagerows': self.__schema['pagerows']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-14.14s = %s%-23.22s%s # %-.60s\n' % ('pagerows',pre,self.__to_string_(value),post,description))
    def __listfile_inp(self):
        out = self.__stdout or sys.stdout
        description = ''
        value = self.__listfile( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'listfile': value},{'listfile': self.__schema['listfile']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-14.14s = %s%-23.22s%s # %-.60s\n' % ('listfile',pre,self.__to_string_(value),post,description))

    #--------- global default implementation-------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def set_global_defaults(self):
        self.set_global_defaults.state['last'] = self
        glb = self.__globals_( )
        if 'antenna' in glb: del glb['antenna']
        if 'pagerows' in glb: del glb['pagerows']
        if 'field' in glb: del glb['field']
        if 'datacolumn' in glb: del glb['datacolumn']
        if 'scan' in glb: del glb['scan']
        if 'vis' in glb: del glb['vis']
        if 'options' in glb: del glb['options']
        if 'array' in glb: del glb['array']
        if 'correlation' in glb: del glb['correlation']
        if 'feed' in glb: del glb['feed']
        if 'showflags' in glb: del glb['showflags']
        if 'uvrange' in glb: del glb['uvrange']
        if 'listfile' in glb: del glb['listfile']
        if 'average' in glb: del glb['average']
        if 'observation' in glb: del glb['observation']
        if 'spw' in glb: del glb['spw']
        if 'selectdata' in glb: del glb['selectdata']
        if 'timerange' in glb: del glb['timerange']


    #--------- inp function -----------------------------------------------------------
    def inp(self):
        self.__vis_inp( )
        self.__options_inp( )
        self.__datacolumn_inp( )
        self.__field_inp( )
        self.__spw_inp( )
        self.__selectdata_inp( )
        self.__antenna_inp( )
        self.__timerange_inp( )
        self.__correlation_inp( )
        self.__scan_inp( )
        self.__feed_inp( )
        self.__array_inp( )
        self.__observation_inp( )
        self.__uvrange_inp( )
        self.__average_inp( )
        self.__showflags_inp( )
        self.__pagerows_inp( )
        self.__listfile_inp( )

    #--------- tget function ----------------------------------------------------------
    def tget(self,file=None):
        from .stack_manip import find_frame
        from runpy import run_path
        filename = None
        if file is None:
            if os.path.isfile("listvis.last"):
                filename = "listvis.last"
        elif isinstance(file, str):
            if os.path.isfile(file):
                filename = file
        if filename is not None:
            glob = find_frame( )
            newglob = run_path( filename, init_globals={ } )
            for i in newglob:
                glob[i] = newglob[i]

    def __call__( self, vis=None, options=None, datacolumn=None, field=None, spw=None, selectdata=None, antenna=None, timerange=None, correlation=None, scan=None, feed=None, array=None, observation=None, uvrange=None, average=None, showflags=None, pagerows=None, listfile=None ):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s
        _prefile = os.path.realpath('listvis.pre')
        _postfile = os.path.realpath('listvis.last')
        _return_result_ = None
        _arguments = [vis,options,datacolumn,field,spw,selectdata,antenna,timerange,correlation,scan,feed,array,observation,uvrange,average,showflags,pagerows,listfile]
        _invocation_parameters = OrderedDict( )
        if any(map(lambda x: x is not None,_arguments)):
            # invoke python style
            # set the non sub-parameters that are not None
            local_global = { }
            if vis is not None: local_global['vis'] = vis
            if options is not None: local_global['options'] = options
            if datacolumn is not None: local_global['datacolumn'] = datacolumn
            if field is not None: local_global['field'] = field
            if spw is not None: local_global['spw'] = spw
            if selectdata is not None: local_global['selectdata'] = selectdata
            if observation is not None: local_global['observation'] = observation
            if average is not None: local_global['average'] = average
            if showflags is not None: local_global['showflags'] = showflags
            if pagerows is not None: local_global['pagerows'] = pagerows
            if listfile is not None: local_global['listfile'] = listfile

            # the invocation parameters for the non-subparameters can now be set - this picks up those defaults
            _invocation_parameters['vis'] = self.__vis( local_global )
            _invocation_parameters['options'] = self.__options( local_global )
            _invocation_parameters['datacolumn'] = self.__datacolumn( local_global )
            _invocation_parameters['field'] = self.__field( local_global )
            _invocation_parameters['spw'] = self.__spw( local_global )
            _invocation_parameters['selectdata'] = self.__selectdata( local_global )
            _invocation_parameters['observation'] = self.__observation( local_global )
            _invocation_parameters['average'] = self.__average( local_global )
            _invocation_parameters['showflags'] = self.__showflags( local_global )
            _invocation_parameters['pagerows'] = self.__pagerows( local_global )
            _invocation_parameters['listfile'] = self.__listfile( local_global )

            # the sub-parameters can then be set. Use the supplied value if not None, else the function, which gets the appropriate default
            _invocation_parameters['antenna'] = self.__antenna( _invocation_parameters ) if antenna is None else antenna
            _invocation_parameters['timerange'] = self.__timerange( _invocation_parameters ) if timerange is None else timerange
            _invocation_parameters['correlation'] = self.__correlation( _invocation_parameters ) if correlation is None else correlation
            _invocation_parameters['scan'] = self.__scan( _invocation_parameters ) if scan is None else scan
            _invocation_parameters['feed'] = self.__feed( _invocation_parameters ) if feed is None else feed
            _invocation_parameters['array'] = self.__array( _invocation_parameters ) if array is None else array
            _invocation_parameters['uvrange'] = self.__uvrange( _invocation_parameters ) if uvrange is None else uvrange

        else:
            # invoke with inp/go semantics
            _invocation_parameters['vis'] = self.__vis( self.__globals_( ) )
            _invocation_parameters['options'] = self.__options( self.__globals_( ) )
            _invocation_parameters['datacolumn'] = self.__datacolumn( self.__globals_( ) )
            _invocation_parameters['field'] = self.__field( self.__globals_( ) )
            _invocation_parameters['spw'] = self.__spw( self.__globals_( ) )
            _invocation_parameters['selectdata'] = self.__selectdata( self.__globals_( ) )
            _invocation_parameters['antenna'] = self.__antenna( self.__globals_( ) )
            _invocation_parameters['timerange'] = self.__timerange( self.__globals_( ) )
            _invocation_parameters['correlation'] = self.__correlation( self.__globals_( ) )
            _invocation_parameters['scan'] = self.__scan( self.__globals_( ) )
            _invocation_parameters['feed'] = self.__feed( self.__globals_( ) )
            _invocation_parameters['array'] = self.__array( self.__globals_( ) )
            _invocation_parameters['observation'] = self.__observation( self.__globals_( ) )
            _invocation_parameters['uvrange'] = self.__uvrange( self.__globals_( ) )
            _invocation_parameters['average'] = self.__average( self.__globals_( ) )
            _invocation_parameters['showflags'] = self.__showflags( self.__globals_( ) )
            _invocation_parameters['pagerows'] = self.__pagerows( self.__globals_( ) )
            _invocation_parameters['listfile'] = self.__listfile( self.__globals_( ) )
        try:
            with open(_prefile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-11s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#listvis( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: pass
        try:
            _return_result_ = _listvis_t( _invocation_parameters['vis'],_invocation_parameters['options'],_invocation_parameters['datacolumn'],_invocation_parameters['field'],_invocation_parameters['spw'],_invocation_parameters['selectdata'],_invocation_parameters['antenna'],_invocation_parameters['timerange'],_invocation_parameters['correlation'],_invocation_parameters['scan'],_invocation_parameters['feed'],_invocation_parameters['array'],_invocation_parameters['observation'],_invocation_parameters['uvrange'],_invocation_parameters['average'],_invocation_parameters['showflags'],_invocation_parameters['pagerows'],_invocation_parameters['listfile'] )
        except: _return_result_ = False
        try:
            os.rename(_prefile,_postfile)
        except: pass
        return _return_result_

listvis = _listvis( )

