##################### generated by xml-casa (v2) from splattotable.xml ##############
##################### 319ac3fb0894daf0222c4393a6d3cea2 ##############################
from __future__ import absolute_import
from casashell.private.stack_manip import find_local as __sf__
from casashell.private.stack_manip import find_frame as _find_frame
from casatools.typecheck import validator as _pc
from casatools.coercetype import coerce as _coerce
from casatasks import splattotable as _splattotable_t
from collections import OrderedDict
import sys
import os

def static_var(varname, value):
    def decorate(func):
        setattr(func, varname, value)
        return func
    return decorate

class _splattotable:
    """
    splattotable ---- Convert a downloaded Splatalogue spectral line list to a casa table.

    
    This task reads a spectral line list(s) downloaded from Splatalogue
    (www.splatalogue.net) and loads it into a CASA table which can be
    queried via eg the slsearch task.

    --------- parameter descriptions ---------------------------------------------

    filenames Files containing Splatalogue lists.
              
              The downloaded files must be in a specific format
              for this task to succeed. Fro details, see the splattotable
              task pages on CASA Docs
              (https://casa.nrao.edu/casadocs/)
    table     Output table name. Must be specified.
    [1;42mRETURNS[1;m      bool

    --------- examples -----------------------------------------------------------

    
    For more information, see the task pages of splattotable in CASA Docs:
    
    https://casa.nrao.edu/casadocs/


    """

    _info_group_ = """analysis"""
    _info_desc_ = """Convert a downloaded Splatalogue spectral line list to a casa table."""

    __schema = {'filenames': {'type': 'cReqPathVec', 'coerce': [_coerce.to_list,_coerce.expand_pathvec]}, 'table': {'type': 'cStr'}}

    def __init__(self):
        self.__stdout = None
        self.__stderr = None
        self.__root_frame_ = None

    def __globals_(self):
        if self.__root_frame_ is None:
            self.__root_frame_ = _find_frame( )
            assert self.__root_frame_ is not None, "could not find CASAshell global frame"
        return self.__root_frame_

    def __to_string_(self,value):
        if type(value) is str:
            return "'%s'" % value
        else:
            return str(value)

    def __validate_(self,doc,schema):
        return _pc.validate(doc,schema)

    #--------- return nonsubparam values ----------------------------------------------

    def __filenames_dflt( self, glb ):
        return [  ]

    def __filenames( self, glb ):
        if 'filenames' in glb: return glb['filenames']
        return [  ]

    def __table_dflt( self, glb ):
        return ''

    def __table( self, glb ):
        if 'table' in glb: return glb['table']
        return ''



    #--------- return inp/go default --------------------------------------------------


    #--------- return subparam values -------------------------------------------------


    #--------- subparam inp output ----------------------------------------------------
    def __filenames_inp(self):
        out = self.__stdout or sys.stdout
        description = 'Files containing Splatalogue lists.'
        value = self.__filenames( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'filenames': value},{'filenames': self.__schema['filenames']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-9.9s = %s%-23.22s%s # %-.60s\n' % ('filenames',pre,self.__to_string_(value),post,description))
    def __table_inp(self):
        out = self.__stdout or sys.stdout
        description = 'Output table name. Must be specified.'
        value = self.__table( self.__globals_( ) )
        (pre,post) = ('','') if self.__validate_({'table': value},{'table': self.__schema['table']}) else ('\x1B[91m','\x1B[0m')
        out.write('%-9.9s = %s%-23.22s%s # %-.60s\n' % ('table',pre,self.__to_string_(value),post,description))

    #--------- global default implementation-------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def set_global_defaults(self):
        self.set_global_defaults.state['last'] = self
        glb = self.__globals_( )
        if 'filenames' in glb: del glb['filenames']
        if 'table' in glb: del glb['table']


    #--------- inp function -----------------------------------------------------------
    def inp(self):
        self.__filenames_inp( )
        self.__table_inp( )

    #--------- tget function ----------------------------------------------------------
    def tget(self,file=None):
        from .stack_manip import find_frame
        from runpy import run_path
        filename = None
        if file is None:
            if os.path.isfile("splattotable.last"):
                filename = "splattotable.last"
        elif isinstance(file, str):
            if os.path.isfile(file):
                filename = file
        if filename is not None:
            glob = find_frame( )
            newglob = run_path( filename, init_globals={ } )
            for i in newglob:
                glob[i] = newglob[i]

    def __call__( self, filenames=None, table=None ):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s
        _prefile = os.path.realpath('splattotable.pre')
        _postfile = os.path.realpath('splattotable.last')
        _return_result_ = None
        _arguments = [filenames,table]
        _invocation_parameters = OrderedDict( )
        if any(map(lambda x: x is not None,_arguments)):
            # invoke python style
            # set the non sub-parameters that are not None
            local_global = { }
            if filenames is not None: local_global['filenames'] = filenames
            if table is not None: local_global['table'] = table

            # the invocation parameters for the non-subparameters can now be set - this picks up those defaults
            _invocation_parameters['filenames'] = self.__filenames( local_global )
            _invocation_parameters['table'] = self.__table( local_global )

            # the sub-parameters can then be set. Use the supplied value if not None, else the function, which gets the appropriate default
            

        else:
            # invoke with inp/go semantics
            _invocation_parameters['filenames'] = self.__filenames( self.__globals_( ) )
            _invocation_parameters['table'] = self.__table( self.__globals_( ) )
        try:
            with open(_prefile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-9s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#splattotable( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: pass
        try:
            _return_result_ = _splattotable_t( _invocation_parameters['filenames'],_invocation_parameters['table'] )
        except: _return_result_ = False
        try:
            os.rename(_prefile,_postfile)
        except: pass
        return _return_result_

splattotable = _splattotable( )

